@@ -24,7 +24,11 @@
 #include <config.h>
 #endif
 
+#include <gal/widgets/e-unicode.h>
+
 #include "gtkhtml.h"
+
+#include "htmlclueflow.h"
 #include "htmlcursor.h"
 #include "htmlengine.h"
 #include "htmllinktext.h"
@@ -48,7 +52,7 @@
 #include "table.h"
 #include "text.h"
 
-#define DEBUG
+/* #define DEBUG */
 #ifdef DEBUG
 #include "gtkhtmldebug.h"
 #endif
@@ -228,6 +232,30 @@ spell_suggest (GtkWidget *mi, GtkHTMLControlData *cd)
 	spell_suggestion_request (cd->html, html_engine_get_word (e), cd);
 }
 
+static void
+spell_check_cb (GtkWidget *mi, GtkHTMLControlData *cd)
+{
+	spell_check_dialog (cd, FALSE);
+}
+
+static void
+spell_add (GtkWidget *mi, GtkHTMLControlData *cd)
+{
+	HTMLEngine *e = cd->html->engine;
+
+	spell_add_to_personal (cd->html, html_engine_get_word (e), cd);
+	html_engine_spell_check (e);
+}
+
+static void
+spell_ignore (GtkWidget *mi, GtkHTMLControlData *cd)
+{
+	HTMLEngine *e = cd->html->engine;
+
+	spell_add_to_session (cd->html, html_engine_get_word (e), cd);
+	html_engine_spell_check (e);
+}
+
 #ifdef DEBUG
 static void
 dump_tree_simple (GtkWidget *mi, GtkHTMLControlData *cd)
@@ -248,14 +276,21 @@ insert_html (GtkWidget *mi, GtkHTMLControlData *cd)
 }
 #endif
 
-#define ADD_ITEM(l,f,t) \
-		menuitem = gtk_menu_item_new_with_label (l); \
+#define ADD_ITEM_BASE(f,t) \
                 gtk_object_set_data (GTK_OBJECT (menuitem), "type", GINT_TO_POINTER (GTK_HTML_EDIT_PROPERTY_ ## t)); \
 		gtk_menu_append (GTK_MENU (menu), menuitem); \
 		gtk_widget_show (menuitem); \
 		gtk_signal_connect (GTK_OBJECT (menuitem), "activate", GTK_SIGNAL_FUNC (f), cd); \
                 (*items)++; items_sep++
 
+#define ADD_ITEM(l,f,t) \
+		menuitem = gtk_menu_item_new_with_label (l); \
+                ADD_ITEM_BASE (f,t)
+
+#define ADD_ITEM_UTF8(l,f,t) \
+                menuitem = e_utf8_gtk_menu_item_new_with_label (GTK_MENU (menu), l); \
+                ADD_ITEM_BASE (f,t)
+
 #define ADD_SEP \
         if (items_sep) { \
                 menuitem = gtk_menu_item_new (); \
@@ -307,17 +342,42 @@ prepare_properties_and_menu (GtkHTMLControlData *cd, guint *items)
 #endif
 
 	if (!html_engine_is_selection_active (e) && obj && html_object_is_text (obj) && !html_engine_word_is_valid (e)) {
-		ADD_SEP;
-		ADD_ITEM (_("Suggest word"), spell_suggest, NONE);
+		gchar *spell, *word, *check_utf8, *add_utf8, *ignore_utf8, *ignore, *add;
+
+		word   = html_engine_get_word (e);
+		check_utf8 = e_utf8_from_locale_string (_("Check '%s' spelling..."));
+		spell  = g_strdup_printf (check_utf8, word);
+		g_free (check_utf8);
+		add_utf8 = e_utf8_from_locale_string (_("Add '%s' to dictionary"));
+		add    = g_strdup_printf (add_utf8, word);
+		g_free (add_utf8);
+		ignore_utf8 = e_utf8_from_locale_string (_("Ignore '%s'"));
+		ignore = g_strdup_printf (ignore_utf8, word);
+		g_free (ignore_utf8);
+		SUBMENU (N_("Spell checker"));
+		if (cd->has_spell_control) {
+			ADD_ITEM_UTF8 (spell, spell_check_cb, NONE);
+		} else {
+			ADD_ITEM (_("Suggest word"), spell_suggest, NONE);
+		}
+		ADD_ITEM_UTF8 (add, spell_add, NONE);
+		ADD_ITEM_UTF8 (ignore, spell_ignore, NONE);
+		END_SUBMENU;
+
+		g_free (spell);
+		g_free (add);
+		g_free (ignore);
+		g_free (word);
 	}
 
-	if (html_engine_is_selection_active (e)
-	    || (obj
-		&& (HTML_OBJECT_TYPE (obj) == HTML_TYPE_LINKTEXT
-		    || (HTML_OBJECT_TYPE (obj) == HTML_TYPE_IMAGE
-			&& (HTML_IMAGE (obj)->url
-			    || HTML_IMAGE (obj)->target))))) {
-		    ADD_ITEM (_("Remove link"), remove_link, NONE);
+	if (cd->format_html && (html_engine_is_selection_active (e)
+				|| (obj
+				    && (HTML_OBJECT_TYPE (obj) == HTML_TYPE_LINKTEXT
+					|| (HTML_OBJECT_TYPE (obj) == HTML_TYPE_IMAGE
+					    && (HTML_IMAGE (obj)->url
+						|| HTML_IMAGE (obj)->target)))))) {
+		ADD_SEP;
+		ADD_ITEM (_("Remove link"), remove_link, NONE);
 	}
 
 	if (html_engine_is_selection_active (e)) {
@@ -331,7 +391,7 @@ prepare_properties_and_menu (GtkHTMLControlData *cd, guint *items)
 		}
 		ADD_ITEM (_("Paste"),  paste, NONE);
 	}
-	if (html_engine_is_selection_active (e)) {
+	if (cd->format_html && html_engine_is_selection_active (e)) {
 		ADD_SEP;
 		ADD_ITEM (_("Text..."), prop_dialog, TEXT);
 		ADD_PROP (TEXT);
@@ -339,7 +399,7 @@ prepare_properties_and_menu (GtkHTMLControlData *cd, guint *items)
 		ADD_PROP (PARAGRAPH);
 		ADD_ITEM (_("Link..."), prop_dialog, LINK);
 		ADD_PROP (LINK);
-	} else if (obj) {
+	} else if (cd->format_html && obj) {
 		switch (HTML_OBJECT_TYPE (obj)) {
 		case HTML_TYPE_RULE:
 			ADD_SEP;
@@ -372,14 +432,16 @@ prepare_properties_and_menu (GtkHTMLControlData *cd, guint *items)
 		default:
 		}
 		if (obj->parent && obj->parent->parent && HTML_IS_TABLE_CELL (obj->parent->parent)) {
-			ADD_SEP;
-			ADD_PROP (CELL);
-			ADD_ITEM (_("Cell..."), prop_dialog, CELL);
-			if (obj->parent->parent->parent && HTML_IS_TABLE (obj->parent->parent->parent)) {
-				ADD_PROP (TABLE);
-				ADD_ITEM (_("Table..."), prop_dialog, TABLE);
+			if (cd->format_html) {
+				ADD_SEP;
+				ADD_PROP (CELL);
+				ADD_ITEM (_("Cell..."), prop_dialog, CELL);
+				if (obj->parent->parent->parent && HTML_IS_TABLE (obj->parent->parent->parent)) {
+					ADD_PROP (TABLE);
+					ADD_ITEM (_("Table..."), prop_dialog, TABLE);
+				}
 			}
-			SUBMENU ("Table insert");
+			SUBMENU (N_("Table insert"));
 			ADD_ITEM (_("Table"), insert_table_cb, NONE);
 			ADD_SEP;
 			ADD_ITEM (_("Row above"), insert_row_above, NONE);
@@ -388,7 +450,7 @@ prepare_properties_and_menu (GtkHTMLControlData *cd, guint *items)
 			ADD_ITEM (_("Column before"), insert_column_before, NONE);
 			ADD_ITEM (_("Column after"), insert_column_after, NONE);
 			END_SUBMENU;
-			SUBMENU ("Table delete");
+			SUBMENU (N_("Table delete"));
 			ADD_ITEM (_("Table"), delete_table, NONE);
 			ADD_ITEM (_("Row"), delete_row, NONE);
 			ADD_ITEM (_("Column"), delete_column, NONE);
@@ -397,9 +459,11 @@ prepare_properties_and_menu (GtkHTMLControlData *cd, guint *items)
 		}
 	}
 
-	ADD_SEP;
-	ADD_PROP (BODY);
-	ADD_ITEM (_("Page..."), prop_dialog, BODY);
+	if (cd->format_html) {
+		ADD_SEP;
+		ADD_PROP (BODY);
+		ADD_ITEM (_("Page..."), prop_dialog, BODY);
+	}
 
 	gtk_widget_show (menu);
 
@@ -410,7 +474,7 @@ gint
 popup_show (GtkHTMLControlData *cd, GdkEventButton *event)
 {
 	GtkWidget *menu;
-	guint items;
+	guint items = 0;
 
 	menu = prepare_properties_and_menu (cd, &items);
 	if (items)
