@@ -44,6 +44,7 @@ struct _GtkHTML {
 
 	GtkBindingSet        *editor_bindings;
 	GtkWidget            *iframe_parent;
+	HTMLObject           *frame;
 	GtkHTMLEditorAPI     *editor_api;
 	gpointer              editor_data;
 	HTMLEngine           *engine;
@@ -72,9 +73,6 @@ struct _GtkHTML {
 	GtkHTMLPrivate *priv;
 };
 
-/* must be forward referenced *sigh* */
-struct _HTMLEmbedded;
-
 struct _GtkHTMLClass {
 	GtkLayoutClass parent_class;
 	
@@ -100,9 +98,10 @@ struct _GtkHTMLClass {
 	void (* iframe_created)     (GtkHTML *html, GtkHTML *iframe);
 
 	/* keybindings signals */
-	void (* scroll)               (GtkHTML *html, GtkOrientation orientation, GtkScrollType scroll_type, gfloat position);
-	void (* cursor_move)          (GtkHTML *html, GtkDirectionType dir_type, GtkHTMLCursorSkipType skip);
-	void (* command)              (GtkHTML *html, GtkHTMLCommandType com_type);
+	void     (* scroll)               (GtkHTML *html, GtkOrientation orientation, GtkScrollType scroll_type,
+					   gfloat position);
+	void     (* cursor_move)          (GtkHTML *html, GtkDirectionType dir_type, GtkHTMLCursorSkipType skip);
+	gboolean (* command)              (GtkHTML *html, GtkHTMLCommandType com_type);
 
 	/* properties */
 	GtkHTMLClassProperties *properties;
@@ -123,6 +122,9 @@ struct _GtkHTMLEditorAPI
 
 	/* input line */
 	GtkWidget * (* create_input_line)     (GtkHTML *html, gpointer data);
+
+	/* spell checking methods */
+	void      (* set_language)            (GtkHTML *html, const gchar *language, gpointer data);
 };
 
 
@@ -139,7 +141,8 @@ void       gtk_html_set_editor_api  (GtkHTML          *html,
 
 /* parent iframe setting */
 void       gtk_html_set_iframe_parent       (GtkHTML *html,
-					     GtkWidget *parent);
+					     GtkWidget *parent,
+					     HTMLObject *frame);
 
 /* Debugging.  */
 void  gtk_html_enable_debug  (GtkHTML  *html,
@@ -154,6 +157,7 @@ void  gtk_html_select_paragraph           (GtkHTML  *html);
 void  gtk_html_select_paragraph_extended  (GtkHTML  *html);
 void  gtk_html_select_all                 (GtkHTML  *html);
 int   gtk_html_request_paste              (GtkHTML  *html,
+					   GdkAtom   selection,
 					   gint      type,
 					   gint32    time);
 /* Loading.  */
@@ -235,6 +239,8 @@ void  gtk_html_paste  (GtkHTML *html);
 void  gtk_html_undo  (GtkHTML *html);
 void  gtk_html_redo  (GtkHTML *html);
 
+void  gtk_html_insert_html (GtkHTML *html, const gchar *html_src);
+
 /* misc utils */
 
 void      gtk_html_set_default_background_color  (GtkHTML     *html,
@@ -243,9 +249,17 @@ void      gtk_html_set_default_content_type      (GtkHTML     *html,
 						  gchar       *content_type);
 gpointer  gtk_html_get_object_by_id              (GtkHTML     *html,
 						  const gchar *id);
-void      gtk_html_editor_command                (GtkHTML     *html,
+gboolean  gtk_html_command                       (GtkHTML     *html,
 						  const gchar *command_name);
 gboolean  gtk_html_edit_make_cursor_visible      (GtkHTML     *html);
 
 gboolean  gtk_html_build_with_gconf              (void);
+void      gtk_html_set_magnification             (GtkHTML *html,
+						  gdouble magnification);
+void      gtk_html_zoom_in                       (GtkHTML *html);
+void      gtk_html_zoom_out                      (GtkHTML *html);
+void      gtk_html_zoom_reset                    (GtkHTML *html);
+
+void      gtk_html_update_styles                 (GtkHTML *html);
+
 #endif /* _GTKHTML_H_ */
